from kashmir.core.data import Id
from kashmir.core.protocols import PProjectRepository


class NewRelease:

    def __init__(self, repository: PProjectRepository):
        self.repository = repository

    def __call__(self, project_id: Id):
        project = self.repository.get(project_id)
        new_release = project.new_release()
        self.repository.create_release(new_release)
        return new_release



from dataclasses import dataclass
from datetime import datetime
from typing import Union

Id = Union[int, str]


@dataclass(frozen=True)
class Version:
    year: int  # AAAA
    release: int  # Non-negative integer
    fix: int  # Non-negative integer

    def __str__(self):
        return f"{self.year}.{self.release}.{self.fix}"

    @classmethod
    def new(cls) -> 'Version':
        return cls(datetime.now().year, 1, 0)


@dataclass(frozen=True)
class Release:
    project: Id
    version: Version

    def __str__(self):
        return f"Release {self.version}"



import click
from kashmir import __version__
from pathlib import Path

from kashmir.providers.gitlab import GitLab
from kashmir.usecases import NewRelease
from kashmir.repositories import ProjectRepository
from kashmir.usecases.new_fix import NewFix


@click.group()
@click.pass_context
def cli(ctx):
    pass


@cli.group('new')
@click.pass_context
def new(ctx):
    pass


@new.command('release')
@click.argument('PROJECT_ID')
@click.option('--scm', default='https://gitlab.com', help='SCM Server address')
@click.option('--token', help='SCM Token', required=True)
@click.pass_context
def new_release(ctx, project_id, scm, token):
    scm = GitLab(
        server=scm,
        token=token,
        project_id=project_id
    )
    repository = ProjectRepository(scm)
    use_case = NewRelease(repository)
    release = use_case(project_id)
    click.echo(f"New release! {release}")


@new.command('fix')
@click.argument('PROJECT_ID')
@click.option('--scm', default='https://gitlab.com', help='SCM Server address')
@click.option('--token', help='SCM Token', required=True)
@click.pass_context
def new_fix(ctx, project_id, scm, token):
    scm = GitLab(
        server=scm,
        token=token,
        project_id=project_id
    )
    repository = ProjectRepository(scm)
    use_case = NewFix(repository)
    release = use_case(project_id)
    click.echo(f"New release! {release}")



from kashmir.core.data import Id, Release
from kashmir.core.entities.project import Project
from kashmir.core.protocols.repository import PProjectRepository
from kashmir.core.providers import SCMProvider


class ProjectRepository(PProjectRepository):

    def __init__(self, scm: SCMProvider):
        self.scm = scm

    def get(self, id: Id) -> Project:
        return Project(
            id=self.scm.project.id,
            version=self.scm.version
        )

    def create_release(self, release: Release) -> Release:
        return self.scm.new_release(release)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['kashmir',
 'kashmir.core',
 'kashmir.core.entities',
 'kashmir.core.protocols',
 'kashmir.core.providers',
 'kashmir.providers',
 'kashmir.repositories',
 'kashmir.usecases']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.0,<8.0', 'python-gitlab>=1.13,<2.0', 'typing-extensions>=3.7,<4.0']

entry_points = \
{'console_scripts': ['kashmir = kashmir:cli']}

setup_kwargs = {
    'name': 'kashmir',
    'version': '2019.1.1',
    'description': "Snow's DevOps CLI Toolkit",
    'long_description': None,
    'author': 'Johnny',
    'author_email': 'johnny.w.santos@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
